#!/usr/bin/perl
use strict;

#
# $Id: FreeBSD.pm,v 1.10 2004/03/18 16:24:28 matt Exp $
#

package MATT::FreeBSD;

use Carp;
use Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION = '1.22';

@ISA     = qw(Exporter);
@EXPORT  = qw(
	StartJail
	CreateJail
	InstallPort
	InstallPackage
	UpdatePortsTree
	UpdateSrcTree
	CheckRcDotConf
);
@EXPORT_OK = qw( 
	IsPackageInstalled
	CheckPortsAge 
);

use lib "lib";
use MATT::Utility;
use MATT::Perl;

=head1 NAME

MATT::FreeBSD - FreeBSD scripting functions

=head1 SYNOPSIS

MATT::FreeBSD is a grouping of frequently used functions I've written for perl scripts running on FreeBSD systems.

=head1 DESCRIPTION

Usage examples for each subroutine are included.

=cut


sub StartJail
{

=head2 StartJail

	use MATT::FreeBSD;
	StartJail($ip, $host, $dir, $debug);

$host is optional. If not passed and reverse DNS is set up, it will
detect the hostname. Otherwise, the hostname defaults to "jail".

$dir is also optional. If not provided, it default to "/home/jail".

Here's an example of how I use it:

	perl -e 'use MATT::FreeBSD; StartJail("10.0.1.175")';

=cut

	my ($ip, $hostname, $dir, $debug) = @_;

	unless ( $dir ) { $dir = "/home/jail"; };

	unless ( $hostname )
	{
		LoadModule("Net::DNS", "p5-Net-DNS", "dns");

    		my $r  = Net::DNS::Resolver->new;

		my $query   = $r->query( $ip, "PTR");

		if ( $query )
    		{
			foreach my $rr ( $query->answer )
			{
				next unless ($rr->type eq "PTR");
				$hostname = $rr->rdatastr;
				print "hostname: " . $rr->rdatastr . " \n";
			};
			if ($hostname =~ /.*\./) 
			{
				($hostname) = $hostname =~ /(.*)\./;
			};
		}
		else
		{
			carp "ns query failed for : ", $r->errorstring if $debug;
			$hostname = "jail";
		};
	};

	print "hostname: $hostname\n";

	chdir("/usr/src");
	unless ( -d "$dir/$ip" ) { croak "The jail dir $dir/$ip doesn't exist!\n" };

	my $mounts = GetMountedDrives($debug);

	unless ( $mounts->{"$dir/$ip/dev"} ) 
	{
		print "mounting $dir/ip/dev\n";
		MATT::Utility::SysCmd("mount_devfs devfs $dir/$ip/dev");
	};

	unless ( $mounts->{"$dir/$ip/proc"} ) 
	{
		print "mounting $dir/ip/proc\n";
		MATT::Utility::SysCmd("mount -t procfs proc $dir/$ip/proc");
	};

	print "starting jail: jail $dir/$ip $hostname $ip /bin/tcsh\n";
	MATT::Utility::SysCmd("jail $dir/$ip $hostname $ip /bin/tcsh");
};

sub CheckRcDotConf
{

=head2 CheckRcDotConf

	use MATT::FreeBSD;
	CheckRcDotConf("snmpd_enable", "snmpd_enable=\"YES\"");

The above example is for snmpd. This checks to verify that an snmpd_enable line exists in /etc/rc.conf. If it doesn't, then it will add it by appending the second argument to the file.

=cut

	my ($check, $line) = @_;

	my $file = "/etc/rc.conf";

	unless ( `grep $check $file | grep -v grep` ) 
	{
		my $r = MATT::Utility::FileAppend($file, $line);
		unless ($r) {
			print "CheckRcDotConf: FAILED to add $line to $file: $!\n";
			print "\n\nNOTICE: It would be a good idea for you to manually add $line to $file.\n\n";
		};
	};
};

sub CreateJail
{

=head2 CreateJail

	use MATT::FreeBSD;
	CreateJail($ip, $host, $dir);

$host is optional. If not passed and reverse DNS is set up, it will
detect the hostname. Otherwise, the hostname defaults to "jail".

$dir is also optional. If not provided, it default to "/home/jail".

Here's an example of how I use it:

	ifconfig fxp0 inet alias 10.0.1.175/32

	perl -e 'use MATT::FreeBSD; CreateJail("10.0.1.175")';

After running CreateJail, you need to set up the jail. 
At the very least, you need to:

    1. set root password
    2. create a user account
    3. get remote root 
        a) use sudo (pkg_add -r sudo; visudo)
        b) add user to wheel group (vi /etc/group)
        c) modify /etc/ssh/sshd_config to permit root login
    4. install perl (pkg_add -r perl)

Here's how I set up my jails:

    pw useradd -n matt -d /home/matt -s /bin/tcsh -m -h 0
    passwd root
    pkg_add -r sudo rsync
    rehash; visudo
    pkg_add -r perl5.8
    sh /etc/rc

Ssh into the jail from another terminal. Once successfully 
logged in with root privs, you can drop the initial shell 
and access the jail directly.

Read the jail man pages for more details. Read the perl code
to see what else it does.

=cut

	my ($ip, $hostname, $dir, $debug) = @_;

	unless ( $dir ) { $dir = "/home/jail"; };

	unless ( $hostname )
	{
		LoadModule("Net::DNS", "p5-Net-DNS", "dns");

    	my $r       = Net::DNS::Resolver->new;
		my $query   = $r->query( $ip, "PTR");

		if ( $query )
    	{
			foreach my $rr ( $query->answer )
			{
				next unless ($rr->type eq "PTR");
				$hostname = $rr->rdatastr;
				print "hostname: " . $rr->rdatastr . " \n";
			};

			if ($hostname =~ /.*\./) 
			{
				($hostname) = $hostname =~ /(.*)\./;
			};
		}
		else
		{
			carp "ns query failed for : ", $r->errorstring if $debug;
			$hostname = "jail";
		};
	};

	# there's probably a better way to reliably do this
	unless ( `ifconfig | grep $ip` ) { 
		croak "Hey! That IP isn't available on any network interface!\n"; };

	unless ( -d "$dir/$ip" ) { MATT::Utility::SysCmd("mkdir -p $dir/$ip"); };
	chdir("/usr/src");

	if ( MATT::Utility::YesOrNo("Do you have a fresh world built?") ) 
	{
		MATT::Utility::SysCmd("make installworld DESTDIR=$dir/$ip");
	} else {
		print "In order to build a jail, you need a fresh world built. That typically means using cvsup to fetch the latest sources from the FreeBSD tree of your choice (I recommend -stable) and then building the world. You can find the instructions for doing this on www.FreeBSD.org. If you already have the FreeBSD source files on your system, you can achieve the desired result by issuing the following command: 

   make -DNOCLEAN world DESTDIR=$dir/$ip
\n";

		if ( MATT::Utility::YesOrNo("Would you like me to do so now?") ) {
			MATT::Utility::SysCmd("make -DNOCLEAN world DESTDIR=$dir/$ip");
		} 
		else { print "Sorry, I cannot continue.\n"; croak; };
	};

	chdir("etc");
	MATT::Utility::SysCmd("make distribution DESTDIR=$dir/$ip");
	MATT::Utility::SysCmd("mount_devfs devfs $dir/$ip/dev");
	chdir("$dir/$ip");
	symlink("dev/null", "kernel");

	mkdir(0755, "$dir/$ip/stand");
	MATT::Utility::SysCmd("cp /stand/sysinstall $dir/$ip/stand");

	MATT::Utility::WriteFile("$dir/$ip/etc/fstab", "");

	my @lines = 'rpcbind_enable="NO"';
	push @lines, 'network_interfaces=""';
	push @lines, 'sshd_enable="YES"';
	push @lines, 'sendmail_enable="NONE"';
	push @lines, 'inetd_enable="YES"';
	push @lines, 'inetd_flags="-wW -a ' . $ip . '"';
	MATT::Utility::WriteFile("$dir/$ip/etc/rc.conf", @lines);

	MATT::Utility::SysCmd("cp /etc/localtime $dir/$ip/etc/localtime");
	MATT::Utility::SysCmd("cp /etc/resolv.conf $dir/$ip/etc/resolv.conf");
	MATT::Utility::FileAppend("$dir/$ip/etc/hosts", "$ip $hostname");
	MATT::Utility::SysCmd("cp /root/.cshrc $dir/$ip/root/.cshrc");
	MATT::Utility::SysCmd("cp /etc/ssl/openssl.cnf $dir/$ip/etc/ssl/openssl.cnf");

	@lines = MATT::Utility::ReadFile("$dir/$ip/etc/ssh/sshd_config");
	foreach my $line (@lines) {
		if ( $line =~ /#ListenAddress 0.0.0.0/ ) {
			$line = "ListenAddress $ip";
		};
	};
	MATT::Utility::WriteFile("$dir/$ip/etc/ssh/sshd_config", @lines);

	MATT::Utility::SysCmd("mount -t procfs proc $dir/$ip/proc");

	if ( MATT::Utility::YesOrNo("Would you like ports installed?") ) 
	{
		my $rsyncbin = MATT::Utility::FindTheBin("rsync");
		unless ($rsyncbin) {
			InstallPackage("rsync");
			$rsyncbin = MATT::Utility::FindTheBin("rsync");
		};

		print "Please be patient, this will take a few minutes (depending on the speed of your disk(s)). \n";

		unless ( -d "$dir/$ip/usr/ports") { mkdir(0755, "$dir/$ip/usr/ports") };

		my @dirs = GetDirFiles("/usr/ports");
		foreach my $pdir (@dirs) 
		{
			next if ($pdir =~ /distfiles$/);
			next if ($pdir =~ /ukrainian$/);
			next if ($pdir =~ /vietnamese$/);
			next if ($pdir =~ /portuguese$/);
			next if ($pdir =~ /polish$/);
			next if ($pdir =~ /russian$/);
			next if ($pdir =~ /korean$/);
			next if ($pdir =~ /hebrew$/);
			next if ($pdir =~ /japanese$/);
			next if ($pdir =~ /hungarian$/);
			next if ($pdir =~ /french$/);
			next if ($pdir =~ /chinese$/);
			print "rsync -aW $pdir $dir/$ip/usr/ports/ \n";
			MATT::Utility::SysCmd("rsync -aW $pdir $dir/$ip/usr/ports/");
			#MATT::Utility::SysCmd("rsync -aW --exclude distfiles /usr/ports $dir/$ip/usr");
		};
	};

	print "You now need to set up the jail. At the very least, you need to:

	1. set root password
	2. create a user account
	3. get remote root 
		a) use sudo (pkg_add -r sudo; visudo)
		b) add user to wheel group (vi /etc/group)
		c) modify /etc/ssh/sshd_config to permit root login
	4. install perl (pkg_add -r perl5.8)

Here's how I set up my jail:

    pw useradd -n matt -d /home/matt -s /bin/tcsh -m -h 0
    passwd root
    pkg_add -r sudo rsync
    rehash; visudo
    pkg_add -r perl5.8
    sh /etc/rc

Ssh into the jail from another terminal. Once successfully logged in with root privs, you can drop the initial shell and manage the jail remotely.

Read the jail man pages for more details.\n\n";

	if ( MATT::Utility::YesOrNo("Do you want to start a shell in the jail?") ) 
	{
		print "starting: jail $dir/$ip $hostname $ip /bin/tcsh\n";
		MATT::Utility::SysCmd("jail $dir/$ip $hostname $ip /bin/tcsh");
	} 
	else 
	{
		print "to run:\n\n\tjail $dir/$ip $hostname $ip /bin/tcsh\n\n";
	};
};

sub UpdateSrcTree
{

=head2 UpdateSrcTree

Updates the FreeBSD sources (/usr/src/*) in preparation for building a fresh FreeBSD world.

	use MATT::FreeBSD;
	UpdateSrcTree($toaster, $method, $cc);

$toaster is the URL to fetch the config file from. It defaults to the Mail::Toaster web site. 

$method is how to call the cvsup process. Options are "fastest" or null. If you choose fastest, then we use the fastest_cvsup port to pick the fastest cvsup server and then pull the sources from that host.

$cc is a country code. Pass it along so that fastest_cvsup narrows the search for cvsup servers to those nearest you.

=cut

	my ($toaster, $method, $conf) = @_;

	my $cvshost = $conf->{'cvsup_server_preferred'};
	my $cc      = $conf->{'cvsup_server_country'};

	unless ($toaster) { $toaster = "http://www.tnpi.biz/internet/mail/toaster"; };
	unless ($method)  { $method  = "fastest"; };

	print "\n\nUpdateSrcTree: Getting ready to update your sources!\n\n";

	my $cvsupbin = MATT::Utility::FindTheBin("cvsup");
	unless ( -x $cvsupbin ) 
	{
		if ( $conf->{'package_install_method'} eq "ports" ) {
			InstallPort("cvsup-without-gui", "net");
		} else {
			InstallPackage("cvsup-without-gui");
		};
	};

	unless ( -e "/usr/local/bin/perl5.6.1" ) {
		if ( -e "/usr/local/bin/perl" ) {
			symlink("/usr/local/bin/perl", "/usr/local/bin/perl5.6.1");
		};
	};

	# some stupid ports think they require perl 5.6.1, this fools them
	# if we have 5.8.1-3 instead.
	unless ( -d "/usr/local/lib/perl5/site_per/5.6.1" ) {
		if ( -d "/usr/local/lib/perl5/site_perl/5.8.2" ) {
			symlink("/usr/local/lib/perl5/site_perl/5.8.2", "/usr/local/lib/perl5/site_per/5.6.1");
		};
		if ( -d "/usr/local/lib/perl5/site_perl/5.8.1" ) {
			symlink("/usr/local/lib/perl5/site_perl/5.8.1", "/usr/local/lib/perl5/site_per/5.6.1");
		};
		if ( -d "/usr/local/lib/perl5/site_perl/5.8.3" ) {
			symlink("/usr/local/lib/perl5/site_perl/5.8.3", "/usr/local/lib/perl5/site_per/5.6.1");
		};
	};

	my $cvsup = $conf->{'cvsup_supfile_sources'};

	unless ( -e $cvsup ) {
		if    ( -e "/etc/cvsup-sources") { $cvsup = "/etc/cvsup-sources"; }
		elsif ( -e "/etc/cvsup-stable" ) { $cvsup = "/etc/cvsup-stable";  } 
		else  
		{ 
			$cvsup = "/etc/cvsup-stable";
			MATT::Utility::FetchFile("$toaster/etc/cvsup-stable");
			move("cvsup-stable", $cvsup);
		};
	};

	if ( $method )
	{
		if ($method eq "fastest" )
		{
			my $fastestbin = MATT::Utility::FindTheBin("fastest_cvsup");

			unless ( $fastestbin) 
			{
				if ( -d "/usr/ports/sysutils/fastest_cvsup" ) 
				{
					InstallPort("fastest_cvsup", "sysutils");
					$fastestbin = MATT::Utility::FindTheBin("fastest_cvsup");
				};
			};

			if ($fastestbin) 
			{
				unless ($cc) {
					$cc = MATT::Utility::GetAnswer("what's your two digit country code?");
				};
				$cvshost = `$fastestbin -Q -c $cc`; chomp $cvshost;
			} 
			else { print "ERROR: fastest_cvsup not installed.\n" };

			unless ($fastestbin) 
			{
				print "ERROR: fastest_cvsup port is not available (yet)\n";
			};
		}
		else { $cvshost = $method };
	};

	if ( $cvshost ) 
	{
		MATT::Utility::SysCmd("$cvsupbin -g -h $cvshost $cvsup");
	} else {
		MATT::Utility::SysCmd("$cvsupbin -g $cvsup");
	};

	print "\n\n
\tAt this point I recommend that you:

\ta) read /usr/src/UPDATING
\tb) make any kernel config options you need
\tc)\tmake buildworld\n\t\tmake kernel\n\t\treboot\n\t\tmake installworld
\t\tmergemaster\n\t\treboot\n\n";

};

sub UpdatePortsTree
{

=head2 UpdatePortsTree

Updates the FreeBSD ports tree (/usr/ports/*).

	use MATT::FreeBSD;
	UpdatePortsTree($toaster, $method, $cc);

$toaster is the URL to fetch the config file from. It defaults to the Mail::Toaster web site. 

$method is how to call the cvsup process. Options are "fastest" or null. If you choose fastest, then we use the fastest_cvsup port to pick the fastest cvsup server and then pull the sources from that host.

$cc is a country code. Pass it along so that fastest_cvsup narrows the search for cvsup servers to those nearest you. If you don't pass it, you'll be prompted to enter it at run time.

=cut

	my ($toaster, $method, $cc) = @_;
	my ($cvshost);

	unless ($toaster) { $toaster = "http://www.tnpi.biz/internet/mail/toaster"; };
	unless ($method)  { $method  = "fastest"; };

	print "\n\nUpdatePorts: You might want to update your ports tree!\n\n";

	if( MATT::Utility::YesOrNo( "\n\nWould you like me to do it for you?:") )
	{   
		my $cvsupbin = MATT::Utility::FindTheBin("cvsup");

		unless ( $cvsupbin && -x $cvsupbin) 
		{
			InstallPackage("cvsup-without-gui");
			$cvsupbin = MATT::Utility::FindTheBin("cvsup");
		};

		my $cvsup = "/etc/cvsup-ports";

		unless ( -e $cvsup )
		{
			MATT::Utility::FetchFile("$toaster/etc/cvsup-ports");
			move("cvsup-ports", $cvsup);
		};

		if ( $method )
		{
			if ($method eq "fastest" )
			{
				my $fastestbin = MATT::Utility::FindTheBin("fastest_cvsup");

				unless ( $fastestbin) 
				{
					if ( -d "/usr/ports/sysutils/fastest_cvsup" ) 
					{
						MATT::FreeBSD::InstallPort("fastest_cvsup", "sysutils");
						$fastestbin = MATT::Utility::FindTheBin("fastest_cvsup");
					};
				};

				if ($fastestbin) 
				{
					unless ($cc) {
						$cc = MATT::Utility::GetAnswer("what's your two digit country code?");
					};
					$cvshost = `$fastestbin -Q -c $cc`; chomp $cvshost;
				} 
				else 
				{ 
					print "ERROR: fastest_cvsup port is not available (yet)\n";
				};
			}
			else { $cvshost = $method };
		};

		if ( $cvshost ) 
		{
			MATT::Utility::SysCmd("$cvsupbin -g -h $cvshost $cvsup");
		} else {
			MATT::Utility::SysCmd("$cvsupbin -g $cvsup");
		};

		if( MATT::Utility::YesOrNo( "\n\nWould you like me to run portsdb -Uu") )
		{
			my $portsdb = MATT::Utility::FindTheBin("portsdb");
			MATT::Utility::SysCmd("$portsdb -Uu");
		};
	};
};


sub InstallPort($$;$$$$$)
{

=head2 InstallPort

	use MATT::FreeBSD;
	InstallPort("net", "openldap2");

That's it. Really. Honest. Nothing more. Well, OK, sometimes it can get a little more complex. InstallPort checks first to determine if a port is already installed and if so, skips right on by. It's very intelligent that way. However, sometimes port maintainers do goofy things and we need to override the directory directory we install from. A good example of this is currently openldap2. 

If you want to install OpenLDAP 2, then you can install from any of:

		/usr/ports/net/openldap2
		/usr/ports/net/openldap20
		/usr/ports/net/openldap21
		/usr/ports/net/openldap22

BTW: The second argument ("net") is what determines where in FreeBSD's ports tree the script can find OpenLDAP. If you pass along a third argument, we'll use it instead of the port name as the port directory to install from.

On rare occasion, a port will get installed as a name other than the ports name. Of course, that wreaks all sorts of havoc so when one of them nasties is found, you can optionally pass along a fourth parameter which can be used as the port installation name to check with.

On yet other occassions, you'll want to pass make flags to the port. The fifth argument can be a comma seperated list of make arguments.

The sixth optional flag is whether errors should be fatal or not. Binary values.

And the seventh is debugging. Setting will increase the amount of logging.

=cut

	my ($name, $base, $booger, $check, $flags, $fatal, $debug) = @_;
	my ($dir, $makef, @defs);

	if ($base eq "net-mgmt" && ! -d "/usr/ports/net-mgmt") {
		$base = "net";
	};

	if ($booger) { $dir = "/usr/ports/$base/$booger" } 
	else         { $dir = "/usr/ports/$base/$name"   };

	CheckPortsAge("30");

	if ($flags) 
	{
		@defs = split(/,/, $flags);
		foreach my $def ( @defs ) 
		{ 
			if ( $def =~ /=/ ) { $makef .= " $def "   } 
			else               { $makef .= " -D$def " };
		};
	};

	$check = $name if (! $check);

	my $pkg_info = MATT::Utility::FindTheBin("pkg_info");
	my $grep     = MATT::Utility::FindTheBin("grep");

	my $r = `$pkg_info | cut -d" " -f1 | $grep $check-`; chomp $r;
	unless ( $r ) 
	{
		$r = `$pkg_info | cut -d" " -f1 | $grep $check`; chomp $r;
	};

	if ( $r )
	{
		printf "InstallPort: %-20s installed as (%s).\n", $name, $r;
		return 0;
	}
	else
	{
		print "InstallPort: installing $name...\n" if $debug;
		chdir($dir) or croak "couldn't cd to $dir: $!\n";
		if ( $name eq "qmail" ) 
		{
			MATT::Utility::SysCmd( "make enable-qmail clean");
			if ( -e "/usr/local/etc/rc.d/qmail.sh" ) 
			{
				use File::Copy;
				move("/usr/local/etc/rc.d/qmail.sh", 
					"/usr/local/etc/rc.d/qmail.sh-dist") or croak "$!";
			};
		} 
		elsif ( $name eq "ezmlm-idx" )
		{
			MATT::Utility::SysCmd( "make $makef install");
			copy("work/ezmlm-0.53/ezmlmrc", "/usr/local/bin");
			MATT::Utility::SysCmd( "make clean");
		} 
		else 
		{
			print "running: make $makef install clean";
			MATT::Utility::SysCmd( "make $makef install clean");
		};
		print "done.\n" if $debug;
	};

	$r = `$pkg_info | cut -d" " -f1 | $grep $check-`; chomp $r;
	unless ( $r ) 
	{
		$r = `$pkg_info | cut -d" " -f1 | $grep $check`; chomp $r;
	};

	if ( $r ne "" )
	{
		printf "InstallPort: %-20s installed as (%s).\n", $name, $r;
		return 1;
	} 
	else 
	{
		printf "InstallPort: %-20s FAILED\n", $name;
		if ($fatal) 
		{
			croak "FATAL FAILURE: Install of $name failed. Please fix and try again.\n";
		} 
		else { return 0; };
	};
};

sub CheckPortsAge($;$)
{
	my ($age, $url) = @_;

=head2 CheckPortsAge

Checks how long it's been since you've updated your ports tree. Since the ports tree can be a roaming target, by making sure it's current before installing ports we can increase the liklihood of success. 

	use MATT::FreeBSD;
	CheckPortsAge("20");

That'll update the ports tree if it's been more than 20 days since it was last updated.

You can optionally pass along a URL as the second argument from where to fetch the cvsup-ports file. If you have a custom one for your site, pass along the URL (minus the file name).

=cut

	$url = "http://www.tnpi.biz/internet/mail/toaster" unless ( $url);

	if ( -M "/usr/ports" > $age )
	{
		UpdatePortsTree();
	}
	else
	{
		print "CheckPortsAge: Ports file is current (enough).\n";
	};
};

sub InstallPackage($;$$$)
{   

=head2 InstallPackage

	use MATT::FreeBSD;
	InstallPackage("ispell");

Suggested usage: 

	unless ( InstallPackage("ispell") ) {
		InstallPort("ispell", "textproc");
	};

Installs the selected package from FreeBSD packages. If the first install fails, it'll try again using an alternate FTP site (ftp2.freebsd.org). If that fails, it returns 0 (failure) so you know it failed and can try something else, like installing via ports.

If the package is registered in FreeBSD's package registry as another name and you want to check against that name (so it doesn't try installing a package that's already installed), instead, pass it along as the second argument.

If you want to retrieve packages from a package site other than FreeBSD's (the default), then pass along that URL as the third argument. See the pkg_add man page for more details.

=cut

	my ($package, $alt, $pkg_url, $debug) = @_;

	my $pkg_add  = MATT::Utility::FindTheBin("pkg_add");

	my $r = IsPackageInstalled($package, $alt);
	if ( $r )
	{
		printf "InstallPackage: %-20s installed as (%s).\n", $package, $r;
		return $r;
	}
	else
	{
		print "InstallPackage: installing $package....\n" if $debug;
		$ENV{"PACKAGESITE"} = $pkg_url if $pkg_url;

		my $r2 = MATT::Utility::SysCmd("$pkg_add -r $package");

		if ($r2) { print "\t pkg_add failed\t "; } 
		else     { print "\t pkg_add success\t " if $debug };

		print "done.\n" if $debug;

		unless ( IsPackageInstalled($package, $alt) ) 
		{
			print "InstallPackage: Failed #1, trying alternate package site.\n";
			$ENV{"PACKAGEROOT"} = "ftp://ftp2.freebsd.org";
			MATT::Utility::SysCmd("$pkg_add -r $package");
		};

		unless ( IsPackageInstalled($package, $alt) ) 
		{
			carp "InstallPackage: Failed again! Sorry, I can't install the package $package!\n";
			return 0;
		};

		return $r;
	};
};

sub IsPackageInstalled
{
	my ($package, $alt) = @_;
	my $r;  

	my $pkg_info = MATT::Utility::FindTheBin("pkg_info");
	my $grep     = MATT::Utility::FindTheBin("grep");

	if ( $alt )
	{
		$r = `$pkg_info | cut -d" " -f1 | $grep $alt-`;
		if ( ! $r ) { $r = `$pkg_info | cut -d" " -f1 | $grep $alt`; };
	} else 
	{
		$r = `$pkg_info | cut -d" " -f1 | $grep $package-`;
		if ( ! $r ) { $r = `$pkg_info | cut -d" " -f1 | $grep $package`; };
	};
	chomp $r;

	return $r;
};


1;
__END__


=head1 AUTHOR

	Matt Simerson <matt@tnpi.biz>


=head1 BUGS

	None known. Report any to author.


=head1 TODO

	Needs more documentation.


=head1 SEE ALSO

	Mail::Toaster, MATT::FreeBSD

	http://www.tnpi.biz/computing/freebsd/
	http://www.tnpi.biz/internet/
	http://www.tnpi.biz/internet/mail/toaster


=head1 COPYRIGHT

Copyright 2003, The Network People, Inc. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the The Network People, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=cut


