#!/usr/bin/perl
use strict;

#
# $Id: Utility.pm,v 1.26 2004/03/15 06:42:05 matt Exp $
#

package MATT::Utility;

use Carp;
use Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION = '1.23';

@ISA   = qw(Exporter);
@EXPORT = qw(
	ArchiveFile
	CdSrcDir
	CleanTmpDir
	DeleteFile
	FetchFile
	FindTheBin
	GetDirFiles 
	GetTheDate
	GetAnswer
	InstallMailToaster
	IsProcessRunning
	LogFileAppend
	ParseConfigFile
	ReadFile
	StripLastDirFromPath 
	SysCmd
	WriteFile
	YesOrNo 
	SudoSetup
);
@EXPORT_OK = qw();

=head1 NAME

MATT::Utility - Common Perl scripting functions


=head1 SYNOPSIS

MATT::Utility is a grouping of frequently used functions I've written.


=head1 DESCRIPTION

Just a big hodge podge of useful subs that I use in scripts all over the place. Peruse through, surely you too can find something of use. 

=cut


sub IsProcessRunning
{

=head2 IsProcessRunning

Verify if a process is running or not.

	use MATT::Utility;
	IsProcessRunning($process);

=cut

	my ($process) = @_;

	my $r = `ps ax | grep $process | grep -v grep`;
	if ( $r ) { return 1; } else { return 0; };
};


sub InstallMailToaster
{

=head2 InstallMailToaster

	use MATT::Utility;
	InstallMailToaster();

Downloads and installs Mail::Toaster.

=cut

	my $src = "/usr/local/src";
	SysCmd("mkdir -p /usr/local/src") unless ( -d $src);
	chdir($src);
	SysCmd("rm -rf Mail-Toaster-*");   # nuke any old versions
	MATT::Utility::FetchFile("http://www.tnpi.biz/internet/mail/toaster/Mail-Toaster.tar.gz");
	SysCmd("tar -xzf Mail-Toaster.tar.gz");

	foreach my $file ( GetDirFiles($src) )
	{
		if ( $file =~ /Mail-Toaster-/ ) 
		{
			chdir($file);
			SysCmd("perl Makefile.PL");
			SysCmd("make test");
			if ( -e "/usr/local/etc/toaster-watcher.conf" ) {
				SysCmd("make conf");
			} else {
				SysCmd("make newconf");
			};
			SysCmd("make cgi");
			SysCmd("make install");
			chdir("..");
			SysCmd("rm -rf $file");
			last;
		};
	};
};

sub files_diff ($$;$$)
{

=head2 files_diff

	use MATT::Utility;
	files_diff($file1, $file2, $type, $debug);

	if ( files_diff("foo", "bar") ) 
	{ 
		print "different!\n"; 
	};

Determine if the files are different. $type is assumed to be text unless you set it otherwise. For anthing but text files, we do a MD5 checksum on the files to determine if they're different or not.

=cut


	my ($f1, $f2, $type, $debug) = @_;

	return -1 unless (-e $f1 && -e $f2 && $type);

	if ($type eq "text") {
		my $diff = SysCmd("diff $f1 $f2");
		return $diff;
	} 
	else 
	{
		use MATT::Perl;
		MATT::Perl::LoadModule("Digest::MD5", "p5-Digest-MD5", "security");

		my @md5sums;
	
		foreach my $f ($f1, $f2) 
		{
			my $ctx = Digest::MD5->new;
			my $sum;
			if(-f "$f.md5"){
				open(F,"$f.md5");
				my $sum=<F>;
				close(F);
				chomp$sum;
			}
			if(!-f "$f.md5" || $sum!~/[0-9a-f]+/i){
				open(F,"$f");
				$ctx->addfile(*F);
				$sum=$ctx->hexdigest;
				close(F);
			}
			push (@md5sums,$sum);
			open(F,">$f.md5");
			print F $sum;
			close(F);
		}

		if ($md5sums[0] eq $md5sums[1]) {
			return 0;
		}
		return 1;
	};
};

sub YesOrNo(;$)
{

=head2 YesOrNo

	YesOrNo ($question)

	use MATT::Utility;

	my $r = YesOrNo("Would you like fries with that?");
	print "yes" if $r;

	Passing $question is optional.

	$r is a scalar

=cut

	my ($question) = @_;
	my $ans;

	print $question if ($question);

	do
	{
		print "(y/n): ";
		$ans = <STDIN>; chomp($ans);  
		$ans = lc($ans); 
	} until ( $ans eq "n" || $ans eq "y" );

	if ($ans eq "y") { return 1; };
	if ($ans eq "n") { return 0; };
};

sub StripLastDirFromPath($) 
{

=head2 StripLastDirFromPath

	StripLastDirFromPath ($dir)

Takes a path like "/usr/home/matt" and returns "/usr/home" and "matt"

You (and I) should be using File::Basename instead as it's more portable.

=cut

	my ($dir) = @_;

	if ( $dir =~ /\/$/ ) { chop $dir };

	my $rindex = rindex($dir, "/");
	my $updir  = substr($dir, 0, $rindex);
	$rindex++;
	my $curdir = substr($dir, $rindex);       

	return $updir, $curdir;
};

sub WriteFile($@)
{

=head2 WriteFile

	WriteFile ($file, @lines)

	use MATT::Utility;

	my $file = "/tmp/foo";
	my @lines = "1", "2", "3";

	print "success" if (WriteFile($file, @lines));

$file is the file you want to write to
@lines is a an array, each array element is a line in the file

A result code is returned, undef is failure, anything else is success.

=cut

	my ($file, @lines) = @_;

	if ( -d $file ) {
		print "WriteFile FAILURE: $file is directory, not a file!\n";
		return undef;
	};

	if ( -f $file && ! -w $file )
	{
		print "WriteFile FAILURE: $file is not writable!\n";
		return undef;
	};

	unless ( open FILE, ">$file" )
	{
		carp "WriteFile: couldn't open $file: $!";
		return undef;
	};

	foreach my $line (@lines)
	{
		print FILE "$line\n";
	};
	close FILE;

	return 1;
};

sub ReadFile($)
{
	my ($file) = @_;

=head2 ReadFile

	use MATT::Utility;
	my @lines = ReadFile($file)

Reads in a file, and returns an array with the files contents, one line per array element. All lines in array have already been chomped.

=cut

	unless ( -r $file ) 
	{
		if ( -e $file ) 
		{
			carp "ReadFile: $file is not readable!\n";
			return 0;
		}
		else
		{
			carp "ReadFile: $file does not exist!\n";
			return 0;
		};
	};

	open(FILE, $file) or carp "ReadFile: couldn't open $file: $!";
	my @lines = <FILE>;
	close FILE;

	chomp @lines;
	return @lines;
};

sub FindTheBin($;$)
{
	my ($bin, $dir) = @_;

=head2 FindTheBin

	use MATT::Utility;
	my $r = FindTheBin($bin, $dir)

Check all the "normal" locations for a binary that should be on the system and returns the full path to the binary. Return zero if we can't find it.

If the optional $dir is sent, then check that directory first and if the binary exists there, returns it's full path.

=cut

	my $prefix = "/usr/local";
  
	if    ( $dir && -x "$dir/$bin"      ) { return "$dir/$bin"; };

	if    ( -x "$prefix/bin/$bin"       ) { return "/usr/local/bin/$bin";  }
	elsif ( -x "$prefix/sbin/$bin"      ) { return "/usr/local/sbin/$bin"; }
	elsif ( -x "$prefix/mysql/bin/$bin" ) { return "$prefix/mysql/bin/$bin"; }
	elsif ( -x "/bin/$bin"              ) { return "/bin/$bin";            }
	elsif ( -x "/usr/bin/$bin"          ) { return "/usr/bin/$bin";        }
	elsif ( -x "/sbin/$bin"             ) { return "/sbin/$bin";           }
	elsif ( -x "/usr/sbin/$bin"         ) { return "/usr/sbin/$bin";       }
	else  { return 0; };
};

sub SysCmd($)
{
	my ($cmd) = @_;

=head2 SysCmd

	use MATT::Utility;
	my $r = SysCmd ($cmd)

	print "SysCmd: error result: $r\n" if ($r != 0);

Just a little wrapper around system calls, that returns any failure codes and prints out the error(s) if present.

=cut

	my $r = system $cmd;
	if ($r != 0)
	{
		print "SysCmd: $cmd\n";
		print "SysCmd: result: $r\n";
	};    
	return $r;
};

sub ArchiveFile($)
{
	my ($file) = @_;

=head2 ArchiveFile

	ArchiveFile ($file)

Make a backup copy of a file by copying the file to $file.timestamp.

=cut

	my $date = time;

	if ( (getpwuid($<))[0] ne "root" )
	{
		my $sudo = FindTheBin("sudo");
		my $cp   = FindTheBin("cp");
		SysCmd( "$sudo cp $file $file.$date");
	} 
	else 
	{
		use File::Copy;
		copy($file, "$file\.$date") or croak "backup of $file to $file.$date failed: $!\n";
	};

	print "ArchiveFile: $file backed up to $file.$date\n";
};

sub GetDirFiles($)
{

=head2 GetDirFiles

	GetDirFiles($dir)

$dir is a directory. The return will be an array of files names contained in that directory.

=cut

	my ($dir) = @_;
	my @files;

	opendir(D, $dir) or croak "GetDirFiles: couldn't open $dir: $!";
	while( defined ( my $f = readdir(D) ) )
	{
		next if $f =~ /^\.\.?$/;
		push @files, "$dir/$f";
	};
	closedir(D);
	return @files;
};

sub CleanTmpDir($)
{
	my ($dir) = @_;

=head2 CleanTmpDir

	CleanTmpDir ($dir)

$dir is a directory. Running this will empty it. Be careful!

=cut

	my @list = GetDirFiles($dir);
	chdir ($dir);
	foreach my $e (@list)
	{
		if ( -f $e )
		{
			DeleteFile($e);
		}
		elsif ( -d $e )
		{
			use File::Path;
			rmtree $e or croak "CleanTmpdir: couldn't delete $e\n";
		};
	};
};

sub DeleteFile($;$)
{       

=head2 DeleteFile

	use MATT::Utility;
	DeleteFile($file, $warn);

=cut

	my ($file, $warn) = @_;

	if ($warn)    
	{
		unlink $file or carp "WARNING: couldn't delete $file: $!\n";
	} else {
		unlink $file or croak "FATAL: couldn't delete $file: $!\n";
	};
};

sub GetTheDate(;$$)
{
	my ($bump, $debug) = @_;

=head2 GetTheDate

	GetTheDate ($bump, $debug)

$bump is the optional offset (in seconds) to subtract from the date.

	use MATT::Utility;

	my ($dd, $mm, $yy, $lm, $hh, $mn, $ss) = GetTheDate();

	$dd = day
	$mm = month
	$yy = year
	$lm = last month
	$hh = hours
	$mn = minutes
	$ss = seconds

=cut

	use MATT::Perl;
	MATT::Perl::LoadModule("Date::Format", "p5-TimeDate", "devel");

	if ($bump) 
	{
		$bump  = $bump * 86400;
		my $ss = Date::Format::time2str("%S", ( time                   ));
		my $mn = Date::Format::time2str("%M", ( time                   ));
		my $hh = Date::Format::time2str("%H", ( time - (        $bump) ));
		my $dd = Date::Format::time2str("%d", ( time - (        $bump) ));
		my $mm = Date::Format::time2str("%m", ( time - (        $bump) ));
		my $yy = Date::Format::time2str("%Y", ( time - (        $bump) ));
		my $lm = Date::Format::time2str("%m", ( time - (2592000+$bump) ));

		print "GetTheDate: $yy/$mm/$dd $hh:$mn\n" if $debug;
		return $dd, $mm, $yy, $lm, $hh, $mn, $ss;
	}
	else
	{
		my $ss = Date::Format::time2str("%S", (time          ));
		my $mn = Date::Format::time2str("%M", (time          ));
		my $hh = Date::Format::time2str("%H", (time          ));
		my $dd = Date::Format::time2str("%d", (time          ));
		my $mm = Date::Format::time2str("%m", (time          ));
		my $yy = Date::Format::time2str("%Y", (time          ));
		my $lm = Date::Format::time2str("%m", (time - 2592000));
		print "GetTheDate: $yy/$mm/$dd $hh:$mn\n" if $debug;
		return $dd, $mm, $yy, $lm, $hh, $mn, $ss;
	};
};

sub FetchFile
{

=head2 FetchFile

	use MATT::Utility;
	FetchFile($url);

Use an appropriate URL fetching utility (fetch, curl, wget, etc) based on your OS to download a file from the $url handed to us. Return a result code of 1 for success, 0 is failure.

=cut


	my ($url) = @_;
	my ($fetchbin, $fetchcmd);

	use POSIX qw(uname);

	if ( (uname)[0] eq "FreeBSD" )
	{
		$fetchbin = FindTheBin("fetch");
		$fetchcmd = "$fetchbin $url";
	}
	elsif ( (uname)[0] eq "Darwin" )
	{
		$fetchbin = FindTheBin("curl");
		$fetchcmd = "$fetchbin -O $url";
	}
	else
	{
		$fetchbin = FindTheBin("wget");
		$fetchcmd = "$fetchbin $url";
	};

	my $r = SysCmd($fetchcmd);

	if ( $r != 0 )
	{
		print "FetchFile error executing $fetchcmd\n";
		print "FetchFile error result:  $r\n";
		return 0;
	};

	return 1;
};

sub GetAnswer
{
	my ($q, $default) = @_;
  
=head2 GetAnswer

	GetAnswer ($q, $default)

$q is the question

$default is an optional default answer.

=cut

	if ($default) { print "Please enter $q. ($default) :" }
	else          { print "Please enter $q: "             };

	my $ans = <STDIN>;
	chomp $ans;

	if ($ans ne "") { return $ans; }
	else
	{
		if ($default) { return $default }
		else          { return 0;       }
	};
};

sub FileAppend($@)
{
	my ($file, @lines) = @_;

=head2 FileAppend

	FileAppend ($file, @lines)

Pass this sub a filename and an array and it'll append the array to the file. It's that simple.

=cut

	unless ( open FILE, ">>$file" )
	{
		carp "FileAppend: couldn't open: $!";
		return 0;
	};

	foreach my $line (@lines)
	{
		print FILE "$line\n";
	};

	close FILE;
	return 1;
};

sub LogFileAppend($@)
{
	my ($file, @lines) = @_;

=head2 LogFileAppend

	LogFileAppend ($file, @lines)

Pass this sub a filename and an array and it'll append a timestamp and the array contents to the file. It's that simple.

=cut

	my ($dd, $mm, $yy, $lm, $hh, $mn, $ss) = GetTheDate();
	my $prog = shift @lines;

	open FILE, ">>$file" or carp "LogFileAppend: couldn't open: $!";
	print FILE "$yy-$mm-$dd $hh:$mn:$ss $prog ";

	foreach my $line (@lines)
	{
		print FILE "$line ";
	};

	print FILE "\n";
	close FILE;
};

sub CdSrcDir($;$)
{
	my ($dir, $src) = @_;

	if ( $src ) 
	{
		if ( ! -e $src )
		{
			mkdir($src, 0755);
			if ( ! -d $src ) 
			{
				print "CdSrcDir: mkdir failed! trying with mkdir -p....";
				SysCmd("mkdir -p $src");
				unless ( -d $src ) { croak "Couldn't create $src.\n"; };
			};
		} 
		else 
		{
			if (! -d $src)
			{
				croak "Something (other than a directory) is at $src and that's my build directory. Please remove it and try again!\n";
			};
		};
	};

	unless ( -d $dir ) { mkdir($dir, 0755) or croak "CdSrcDir: FAILED to create $src\n"; };

	chdir($dir);
}; 

sub SourceWarning($;$$)
{
	my ($package, $clean, $src) = @_;

	$src = "/usr/local/src" if ! $src;

	print "\n$package sources are already present, indicating that you've already\n";
	print "installed $package. If you want to reinstall it, remove the existing\n";
	print "sources (rm -r $src/mail/$package) and re-run this script\n\n";

	if ($clean) 
	{
		if ( YesOrNo("\n\tWould you like me to remove the sources for you? ") )
		{
			print "Deleting $package...";
			rmtree $package;
			print "done.\n";
			return 1;
		} 
		else 
		{
			return 0;
		};
	} 
	else 
	{
		return 0;
	};
};

sub ParseConfigFile($;$$)
{
	my ($file, $debug, $etcdir) = @_;

=head2 ParseConfigFile

	use MATT::Utility;
	my $vals = ParseConfigFile ( "example.conf", 1);

	ParseConfigFile ($file, $debug, $etcdir)

$file is the file to be parsed. $etcdir is where the file should be found. It defaults to /usr/local/etc and will also check the current working directory. 

A hashref is returned with the key/value pairs.

=cut

	unless ( $etcdir )
	{
		$etcdir = "/usr/local/etc"; 
	};

	if ( -r "$etcdir/$file" )
	{
		$file = "$etcdir/$file" 
	} 
	else 
	{
		if ( -r "./$file" )
		{
			$file = "./$file" 
		} 
		else { $file = "./$file-dist" if ( -r "./$file-dist" ); };
	};

	if ( -r $file) 
	{
		my (%array);

		print "ParseConfigFile: $file\n" if ($debug);
		open(CONFIG, $file) or carp "WARNING: Can't open $file: $!";

		while ( <CONFIG> ) 
		{
			chomp;
			next if /^#/;       # skip lines beginning with #
			next if /^\s*$/;    # skip empty lines
			print "$_ \n" if $debug;

			# this regexp must match and return these patterns
			# localhost1  = localhost, disk, da0, disk_da0
			# htmldir = /usr/local/rrdutil/html
			# hosts   = localhost lab.simerson.net seattle.simerson.net

			#my $line = $_;
			#$line =~ /(\w+)\s*=\s*(.*)\s*$/;

			my ($key, $val) = $_ =~ /\s*(.*?)\s*\=\s*(.*\S)\s*$/;
			if ( $val =~ /#/ )    { ($val) = $val =~ /(.*?\S)\s*#/ };

			print "$key. \t\t = .$val. \n" if $debug;

			#$array{$key} = $val if $key;
			$array{$key} = $val;
		};

		close(CONFIG);
		return \%array;
	} 
	else 
	{
		carp "WARNING: ParseConfigFile: can't read $file!\n";
		return 0;
	};
};

sub SudoSetup()    
{

=head2	SudoSetup

	use MATT::Utility;
	my $sudo = SudoSetup();

	$r = SysCmd("$sudo mv /tmp/foo /etc/root-owned-file");

Often you want to run a script as an unprivileged user. However, the script may need elevated privileges for a plethora of reasons. Rather than running the script suid, or as root, configure sudo allowing the script to run system commands with appropriate permissions.

=cut

	my $sudo;
	my $sudobin = FindTheBin("sudo");

	if (! $sudobin )
	{
		if ( $< eq 0 )
		{
			use POSIX qw(uname);

			if ( (uname)[0] eq "FreeBSD" )
			{
				if ( YesOrNo("sudo is not installed, shall I install it?" ) )
				{
					use MATT::FreeBSD;
					MATT::FreeBSD::InstallPort("sudo", "security");
					$sudobin = FindTheBin("sudo");

					if (! $sudobin) { croak "sudo installation failed!\n"; };
				}
				else
				{    
					print "very well then, continuing.\n";
					return $sudo;
				};
			} 
			else 
			{
				print "WARNING: Sudo is not installed. Some features will not \n";
				print "work for non-root users.\n";
				return $sudo;
			};
		} 
		else 
		{
			print "\n\n\tWARNING: Couldn't find sudo. Some features require root ";
			print "permissions and will not work without it. If it's installed, ";
			print "please report it's location to Matt so future versions of ";
			print "this script will know where to find it. \n\n";
		};
	};

	if ( $< ne "0" ) 
	{
		$sudo = "$sudobin -p 'Password for %u@%h:'";
	};

	return $sudo;
};


1;
__END__


=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to author.

=head1 TODO

=head1 SEE ALSO

	http://www.tnpi.biz/computing/perl/
	http://www.tnpi.biz/internet/

    MATT::Apache, MATT::DNS, MATT::FreeBSD, MATT::Mysql,
    MATT::Passwd, MATT::Perl, MATT::Qmail, MATT::Quota
    MATT::SSL, MATT::Utility


=head1 COPYRIGHT

Copyright 2003, The Network People, Inc. All Rights Reserved.

=cut
